#http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/252508
""" unzip.py
    Version: 1.1

    Extract a zipfile to the directoryectory provided
    It first creates the directory structure to house the files
    then it extracts the files to it.

    Sample usage:
    command line
    unzip.py -p 10 -z c:\testfile.zip -o c:\testoutput

    python class
    import unzip
    un = unzip.unzip()
    un.extract(r'c:\testfile.zip', 'c:\testoutput')
    

    By Doug Tolton
"""

import sys
import zipfile
import os
import os.path
import getopt

class unzip:
    def __init__(self, verbose = False, percent = 10):
        self.verbose = verbose
        self.percent = percent
        
    def extract(self, file, directory):
        if not directory.endswith(':') and not os.path.exists(directory):
            os.mkdir(directory)

        zf = zipfile.ZipFile(file)

        # create directoryectory structure to house files
        self._createstructure(file, directory)

        num_files = len(zf.namelist())
        percent = self.percent
        divisions = 100 / percent
        perc = int(num_files / divisions)

        # extract files to directoryectory structure
        for i, name in enumerate(zf.namelist()):
            if self.verbose == True:
                print "Extracting %s" % name
            elif perc > 0 and (i % perc) == 0 and i > 0:
                complete = int (i / perc) * percent
                print "%s%% complete" % complete

            if not name.endswith('/'):
                if os.name == 'nt': #sys.platform =='win32':
                    name1 = name.replace("/","\\")
                else:
                    name1 = name
                outfile = open(os.path.join(directory, name1), 'wb')
                outfile.write(zf.read(name))
                outfile.flush()
                outfile.close()


    def _createstructure(self, file, directory):
        self._makedirs(self._listdirs(file), directory)


    def _makedirs(self, directories, basedirectory):
        """ Create any directoryectories that don't currently exist """
        for directory in directories:
            if os.name == 'nt': #sys.platform=='win32':
                dirlist = directory.split('\\')
            else:
                dirlist = directory.split('/')
            curdirectory = basedirectory
            for direct in dirlist:
                newdir = os.path.join(curdirectory, direct)
                if not os.path.exists(newdir):
                    try:
                        os.mkdir(newdir)
                    except OSError:
                        head, tail = os.path.split(newdir)
                        os.mkdir(head)
                        os.mkdir(newdir)
                curdirectory = newdir

                
    def _listdirs(self, file):
        """ Grabs all the directoryectories in the zip structure
        This is necessary to create the structure before trying
        to extract the file to it. """
        zf = zipfile.ZipFile(file)

        directories = {}

        for name in zf.namelist():
            direct, filename = os.path.split(name)
            if os.name == 'nt': #sys.platform=='win32':
                direct = direct.replace('/', '\\')
            try:
                a = directories[direct]
            except KeyError:
                directories[direct] = 1
                
            #if sys.platform == 'win32':
            #    if name.endswith('\\'):
            #        directorys.append(name)
            #elif name.endswith('/'):
            #    directorys.append(name)
        directories = directories.keys()
        directories.sort()
        return directories

def usage():
    print """usage: unzip.py -z <zipfile> -o <targetdirectory>
    <zipfile> is the source zipfile to extract
    <targetdirectory> is the target destination

    -z zipfile to extract
    -o target location
    -p sets the percentage notification
    -v sets the extraction to verbose (overrides -p)

    long options also work:
    --verbose
    --percent=10
    --zipfile=<zipfile>
    --outdirectory=<targetdirectory>"""
    

def main():
    shortargs = 'vhp:z:o:'
    longargs = ['verbose', 'help', 'percent=', 'zipfile=', 'outdir=']

    unzipper = unzip()

    try:
        opts, args = getopt.getopt(sys.argv[1:], shortargs, longargs)
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    zipsource = ""
    zipdest = ""

    for o, a in opts:
        if o in ("-v", "--verbose"):
            unzipper.verbose = True
        if o in ("-p", "--percent"):
            if not unzipper.verbose == True:
                unzipper.percent = int(a)
        if o in ("-z", "--zipfile"):
            zipsource = a
        if o in ("-o", "--outdir"):
            zipdest = a
        if o in ("-h", "--help"):
            usage()
            sys.exit()

    if zipsource == "" or zipdest == "":
        usage()
        sys.exit()
            
    unzipper.extract(zipsource, zipdest)

if __name__ == '__main__': main()



import os, sys
import time
import stat
import string

mglpacks=[ "DejaVu", "MolKit", "ViewerFramework",  "Pmv", "AutoDockTools", "PyBabel",
           "symserv", "mglutil", "NetworkEditor", "Volume", "Vision","PyAutoDock", "Support",
           "Scenario2"]                              
    
def getTestsDirs(file_list, directory, names):
    if os.path.basename(directory) == "Tests":
        file_list.append(os.path.dirname(directory))
        
def getTestSummaryResults(testroot, testreportdir):
    """Parse the test report files created by tester.
    testroot - directory containing the MGLTools packages ;
    testreportdir - directory containing testreports """
    #print "testreportdir:", testreportdir
    assert os.path.exists(testreportdir)
    assert os.path.exists(testroot)   
    # list of directories containing Tests ( created when the tests ran last for this platform)

    cwd = os.getcwd()
    
    testdirs = []
    os.chdir(testroot)
    for p in mglpacks :
        os.path.walk(p, getTestsDirs, testdirs)
    #print  "testdirs" ,testdirs
    os.chdir(cwd)
    reports = []
    for p in testdirs:
        reports.append(p.replace(os.sep,'.'))

    # a dictionary containing the names of test reports
    rantests = {}
    # a dictionary containing the names of test reports (with -s option)
    rantestsS = {}
    # a list of test modules for which we did not get a report
    noreports= []

    modules = []
    # string containing time when the directory was last modified
    reporttime = time.asctime(time.localtime(os.stat(testreportdir)[stat.ST_MTIME])) 

    # remove time (##:##:##)   from reporttime string (leave only the date)
    rd = string.split(reporttime)
    rd.pop(-2)
    reportdate = "%s %s %s %s" % tuple(rd)


    import sys
    for r in reports:
        f1 = os.path.join(testroot, testreportdir,
                          "report-" + r + sys.platform + ".txt")
        f2 = os.path.join(testroot, testreportdir,
                          "sReport-"+ r + sys.platform + ".txt")

        if os.path.exists(f1):
            rantests[f1] = r 
        elif os.path.exists(f2):
            rantestsS[f2] = r
        else:
            noreports.append(r)

    numtests = 0
    numerrors = 0
    results = {}
    for file in rantestsS.keys():
        test = None
        ntests = 0
        testerrors = 0
        f= open(file, "r")
        st =f.readline()
        l = string.split(st)
        if len(l)== 5:
            if l[0] == "RAN":
                test = rantestsS[file]
                ntests = float(l[1])
                numtests = numtests + ntests
                modules.append(test)
                results[test]={}
                results[test]['ntests']= ntests
        txt = f.readlines()
        f.close()
        testname = None
        errlist = []
        errtxt = ""
        for line in txt:
            if string.find(line, "Tests.test_")> 0:
                if string.find(line, "ERROR:")== 0 or string.find(line, "FAIL:")==0:
                    errlist.append(line)
                    continue
                testname =  line
                if len(errlist) >0:
                    errtxt = errtxt + "\n" + testname
                    for err in errlist:
                        errtxt = errtxt + err
                    testerrors = testerrors +len(errlist)
                    errlist = []
                
                continue
            elif string.find(line, "ERROR:")== 0 or string.find(line, "FAIL:")==0:
                errlist.append(line)
            elif line.startswith("RAN "):
                if test == None:
                    test = rantestsS[file]
                    modules.append(test)
                    results[test]={}
                #print test, file, ntests, numtests,
                nt = float(line.split()[1])
                ntests = ntests+nt
                numtests = numtests + nt
                #print ntests, numtests 
                results[test]['ntests']= ntests
                
        #this is for the case when error is in the last test:       
        if len(errlist) >0:
            errtxt = errtxt + "\n" + testname
            for err in errlist:
                errtxt = errtxt + err
            testerrors = testerrors +len(errlist)
            errlist = []

        if testerrors:
            numerrors = numerrors + testerrors
            results[test]['failed'] = testerrors
            results[test]['errtxt'] = errtxt

    for file in rantests.keys():
        testerrors = 0
        f= open(file, "r")
        st =f.readline()
        l = string.split(st)
        if len(l)== 5:
            if l[0] == "RAN":
                test = rantests[file]
                modules.append(test)
                ntests = float(l[1])
                numtests = numtests + ntests
                results[test]={}
                results[test]['ntests']= ntests

        txt = f.readlines()
        f.close()
        testname = None
        errlist = []
        errtxt = ""
        for line in txt:
            if string.find(line, "FAILED (")==0:
                if len(errlist) >0:
                    for err in errlist:
                        errtxt = errtxt + err
                    testerrors = testerrors +len(errlist)
                    errlist = []
                continue
            elif string.find(line, "ERROR:")== 0 or string.find(line, "FAIL:")==0:
                errlist.append(line)
        if testerrors:
            numerrors = numerrors + testerrors
            results[test]['failed'] = testerrors
            results[test]['errtxt'] = errtxt

    perc = numerrors/numtests*100.0
    if len(noreports):
        endstr = "No test report for the following module(s) (tests possibly crashed): \n"
        for f in noreports:
            endstr = endstr+ "%s \n" % f
        results['tail']= endstr   
    results['reportdate'] = reportdate
    results['numtests'] = numtests
    results['numerrors'] = numerrors
    results['perc'] = perc

    return results, modules



def mkSummaryHTML(testroot, testreportdir, filename = None):
    """ create a simple html file with the tests summary"""
    
    results, modules = getTestSummaryResults(testroot, testreportdir)
    if not filename:
        if sys.platform == "darwin":
            if os.uname()[-1] == 'Power Macintosh':
                prt = "powerpc"
            else:
                prt = "i386"
            filename = "testsummary"+sys.platform+ prt+".html"
        else:
            filename = "testsummary"+sys.platform+".html"
        
    resfile = open(filename, "w")
    
    txt = """
    <!-- <html>
    <head>
    <title>TestSummary</title>
    </head>
    <body bgcolor=#ffffff text=#000000> -->
    <html>
    <BODY text=#000000 bgcolor=#FFFFFF >
    <font color=#7c0027 size=+1>Summary of Tests Results ran on %s.</font>
    <br> """ % results['reportdate']
    resfile.write(txt)
    for mod in modules:
        ntests = results[mod]['ntests']
        if ntests == 0:
            continue
        #resfile.write("<b>%s</b><br> \n" % mod)
        resfile.write("<b>%s &nbsp; &nbsp; </b>\n" % mod+"   ")
        
        if results[mod].has_key("failed"):
            nfailed = results[mod]["failed"]
            #txt = "<pre>\nRan <b>%d</b> tests: number of failed tests <b>%d</b> \nThe following tests failed: \n" % (ntests, nfailed)
            txt = "Ran <b>%d</b> tests; <b>%d</b> test(s) failed:\n" % (ntests, nfailed)
            resfile.write(txt)
            resfile.write("<pre>")
            resfile.write(results[mod]['errtxt']+ "</pre> \n" )
            
            #resfile.write("</pre>\n-----------------------------------------------------------<br>\n")
        else:
            resfile.write("Ran <b>%d</b> tests: OK <br>\n" % ntests)
            #resfile.write("-----------------------------------------------------------<br>\n")

    resfile.write("===========================================================<br>\n")
    resfile.write("<b>Overall Results(%s): ran %d tests, failed %d (%.2f %s) </b><br> \n" %
                  (results['reportdate'], results['numtests'], results['numerrors'], results['perc'], "%") )
    if results.has_key('tail'):
        resfile.write("<pre>\n%s</pre>" % results['tail'])
    
    resfile.close()
    

if __name__ == "__main__":

    args = sys.argv[1:]
    #print "args" ,args
    if len(args) == 2:
        mkSummaryHTML(args[0], args[1])
    elif len(args) == 3:
        mkSummaryHTML(args[0], args[1], args[2])
    else:
        print "mkreport requires 2 or 3 arguments"




# specify mglroot here
import sys, os
path = os.path.join(mglroot, "MGLToolsPckgs")
sys.path.append(path)

from os import getenv
if getenv('MGLPYTHONPATH'):
    sys.path.insert(0, getenv('MGLPYTHONPATH'))
    
from Support.path import setSysPath
setSysPath(path)
#sys.path.insert(0,os.path.abspath('.'))
